/*--------------------------------------------------------------------------
 *  Copyright 2008 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// utgb-core Project
//
// UTGBEntryPointBase.java
// Since: Jun 2, 2008
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/utgb/client/UTGBEntryPointBase.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.gwt.utgb.client;

import java.util.HashMap;
import java.util.Map;

import org.utgenome.gwt.utgb.client.track.TrackGroup;
import org.utgenome.gwt.utgb.client.track.TrackLoader;
import org.utgenome.gwt.utgb.client.track.TrackQueue;
import org.utgenome.gwt.utgb.client.util.BrowserInfo;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.http.client.Request;
import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestCallback;
import com.google.gwt.http.client.RequestException;
import com.google.gwt.http.client.Response;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.RootPanel;

public abstract class UTGBEntryPointBase implements EntryPoint {
	// widgets
	private DockPanel dockPanel = new DockPanel();
	private final TrackGroup trackGroup = new TrackGroup("root");
	private final TrackQueue trackQueue = new TrackQueue(trackGroup);

	private HashMap<String, String> queryParam = new HashMap<String, String>();

	public TrackGroup getTrackGroup() {
		return trackGroup;
	}

	public TrackQueue getTrackQueue() {
		return trackQueue;
	}

	public void onModuleLoad() {
		GenomeBrowser.initServices();
		queryParam = BrowserInfo.getURLQueryRequestParameters();

		// invoke main method
		main();
	}

	public void displayTrackView() {
		// load a view
		if (queryParam.containsKey("view")) {
			loadView(queryParam.get("view"));
		}
		else {
			loadView("default-view.xml");
		}

		dockPanel.add(trackQueue, DockPanel.CENTER);
		RootPanel rootPanel = RootPanel.get("utgb-main");
		if (rootPanel != null) {
			rootPanel.add(dockPanel);
		}
		else {
			RootPanel.get().add(new Label("Error: <div id=\"utgb-main\"></div> tag is not found in this HTML file."));
		}
	}

	/**
	 * load the view XML file from the public/view folder.
	 * 
	 * @param viewXMLPath
	 */
	public void loadView(String viewXMLURL) {
		if (viewXMLURL.startsWith("http://")) {
			GenomeBrowser.getService().getHTTPContent(viewXMLURL, new AsyncCallback<String>() {
				public void onFailure(Throwable arg0) {
					GWT.log("view retrieval failed", null);
				}

				public void onSuccess(String view) {
					updateView(view);
				}
			});
		}
		else {
			String url = GWT.getModuleBaseURL() + "view/" + viewXMLURL;
			RequestBuilder viewGetRequest = new RequestBuilder(RequestBuilder.GET, url);
			try {
				viewGetRequest.sendRequest(null, new RequestCallback() {
					public void onError(Request arg0, Throwable arg1) {
						GWT.log("view retrieval failed", null);
					}

					public void onResponseReceived(Request req, Response resp) {
						updateView(resp.getText());
					}
				});
			}
			catch (RequestException e) {
				GWT.log(e.getMessage(), e);
			}
		}

	}

	private void updateView(String viewXML) {
		TrackGroup newGroup = TrackLoader.createTrackGroupFromXML(viewXML);
		// apply the URL query parameters
		trackGroup.addTrackGroup(newGroup);
		newGroup.getPropertyWriter().setProperty(queryParam);

		if (queryParam.containsKey("start")) {
			int start = Integer.parseInt(queryParam.get("start"));
			int end = queryParam.containsKey("end") ? Integer.parseInt(queryParam.get("end")) : start + 1000;
			newGroup.getPropertyWriter().setTrackWindow(start, end);
		}
	}

	public Map<String, String> getQueryParameterTable() {
		return queryParam;
	}

	public abstract void main();

}
