/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// GenomeBrowser Project
//
// DatabaseService.java
// Since: Jun 29, 2007
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/utgb/client/db/DatabaseService.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.gwt.utgb.client.db;

import java.util.List;

import org.utgenome.gwt.utgb.client.track.bean.LegacyTrackInformation;

import com.google.gwt.user.client.rpc.RemoteService;

/**
 * Distributed Database Server Service
 * 
 * @author leo
 * 
 */
public interface DatabaseService extends RemoteService {

	/**
	 * Queries database data
	 * 
	 * @param jsonQuery
	 *            the query generted from {@link Query#toJSON}
	 * @return the query result
	 * @see {@link Query}
	 */
	public String query(String path, String jsonQuery);

	// file server data

	/**
	 * Gets list of the database catalog data hold by this database server
	 * 
	 * @return JSON representation of database catalogs
	 */
	public String getCatalog();

	/**
	 * Gets the list of database server locations (IP addresses) of the given path
	 * 
	 * @return the list of database servers (JSON format)
	 */
	public List<String> getDatabaseLocation(String path);

	/**
	 * Gets the list of available species
	 * 
	 * @return the list of species
	 */
	public List<String> getAvailableSpecies();

	/**
	 * Gets the list of sequence revisions for the species
	 * 
	 * @return the list of sequence revisions
	 */
	public List<String> getSequenceRevision(String species);

	/**
	 * Gets the list of Legacy Tracks
	 * 
	 * @return the list of TrackBeans
	 */
	public List<LegacyTrackInformation> getLegacyTrackList(String species, String sequenceRevision);

}
