/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// GenomeBrowser Project
//
// GenomeViewer.java
// Since: Jun 18, 2007
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/utgb/client/track/GenomeViewer.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.gwt.utgb.client.track;

import java.util.HashMap;

import org.utgenome.gwt.utgb.client.track.impl.TrackWindowImpl;
import org.utgenome.gwt.utgb.client.ui.CSS;
import org.utgenome.gwt.utgb.client.util.BrowserInfo;
import org.utgenome.gwt.web.client.tab.TabEntry;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.HTTPRequest;
import com.google.gwt.user.client.ResponseTextHandler;
import com.google.gwt.user.client.ui.DockPanel;

public class GenomeViewer extends TabEntry {
	public static TabEntryFactory factory() {
		return new TabEntryFactory("Genome Viewer", "UTGB Genome Browser") {
			public TabEntry createInstance() {
				return new GenomeViewer();
			}
		};
	}

	private final DockPanel _panel = new DockPanel();
	private TrackGroup _globalTrackGroup = new TrackGroup("Genome Viewer");
	private final TrackQueue _queue = new TrackQueue(_globalTrackGroup);

	public GenomeViewer() {
		init();
	}

	public void init() {
		HashMap<String, String> query = BrowserInfo.getURLQueryRequestParameters();
		// load a view
		if (query.containsKey("view")) {
			String url = GWT.getModuleBaseURL() + "view/" + query.get("view");
			HTTPRequest.asyncGet(url, new ResponseTextHandler() {
				public void onCompletion(String responseText) {
					TrackGroup newGroup = TrackLoader.createTrackGroupFromXML(responseText);
					// _globalTrackGroup.loadFromXML(responseText);
				}
			});
		}
		HashMap<String, String> selectedParam = new HashMap<String, String>();
		String[] param = { "species", "revision", "target" };
		for (int i = 0; i < param.length; i++) {
			if (query.containsKey(param[i]))
				selectedParam.put(param[i], query.get(param[i]));
		}
		_globalTrackGroup.getPropertyWriter().setProperty(selectedParam);
		_globalTrackGroup.setTrackWindow(new TrackWindowImpl(600, 0, 1000000));
		_panel.add(_queue, DockPanel.CENTER);
		CSS.fullSize(_panel);
		initWidget(_panel);
	}

	public void onFocus() {
	}

	public void onLostFocus() {
	}
}
